/**
 * @file		RSPTerrain_datatypes.h
 * @date		11/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		RSP Terrain workflow structures definitions
 *
 */

#ifndef RSPTERRAINLIB_DATATYPES_H_
#define RSPTERRAINLIB_DATATYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

// Number of height values to take for computing terrain (default: 4)
#define RSPTERRAINLIB_TILE_SAMPLING		4

#if (RSPTERRAINLIB_TILE_SAMPLING > 5 || RSPTERRAINLIB_TILE_SAMPLING < 1)
#error "RSPTERRAINLIB_TILE_SAMPLING must be within the range of 1-5."
#endif

///////////////////////////////////////////////////////////////////////////////
//                         Configuration structure
///////////////////////////////////////////////////////////////////////////////

typedef union u_rspterrain_parameters {
	struct {
		unsigned char verbose_mode:1;	//!< Output simple details about ID and other "light" things.
		unsigned char debug_mode:1;		//!< Output all values of faces, indices and vertices and others "heavy" things.
		unsigned char god_mode:1;		//!< Dev only. Output experimental values.

		unsigned char reserved0:1;		//!< For future use.

		unsigned char invertZ:1;		//!< Invert Z axis on tilesmap output.
		unsigned char negative:1;		//!< Enable negative tilesmap output.

		unsigned char reserved1:2;		//!< For future use.
	};
	unsigned char raw;	//!< Raw options access for bit-masking or memory copy/compare.
} RSPTERRAIN_PARAMETERS ;


////////////////////////////////////////////////////////////////////////////////
//                         Lib's structure definitions
////////////////////////////////////////////////////////////////////////////////

#ifndef MEMFILE_DEF
typedef char* MEMFILE;
#define MEMFILE_DEF
#endif

#ifndef HEIGHTMAP_T_DEF
typedef unsigned char** HEIGHTMAP_T;
#define HEIGHTMAP_T_DEF
#endif

#ifndef T_VECTOR3_DEF
typedef struct vector3 { float x,y,z; } T_VECTOR3;
#define T_VECTOR3_DEF
#endif

#ifndef T_VERTEX_DEF
typedef T_VECTOR3 T_VERTEX;
#define T_VERTEX_DEF
#endif

typedef struct rspterrain_heightmap {
	unsigned short width;
	unsigned short height;

	HEIGHTMAP_T heightmap;
} T_RSPTERRAIN_HEIGHTMAP ;

typedef struct rspterrain_mesh {
	unsigned short width;
	unsigned short height;

	unsigned int vertices_count;
	T_VERTEX* verticesmap;
} T_RSPTERRAIN_MESH ;

typedef struct rspterrain_tile {
	unsigned short texmap_id;

	unsigned char low_height;
	unsigned char high_height;

	unsigned char height_values[5][5];
} T_RSPTERRAIN_TILE;

typedef struct rspterrain_obj {
	unsigned short width;
	unsigned short height;

	float height_scale;

	unsigned short tiles_count;
	T_RSPTERRAIN_TILE* tilesmap;
} T_RSPTERRAIN_HMP ;

#ifdef __cplusplus
}
#endif

#endif /* RSPTERRAINLIB_DATATYPES_H_ */
